#include "hbclass.ch"
#include "common.ch"

#xcommand REPEAT => WHILE .T.
#xcommand UNTIL <exp> => IF <exp> ; EXIT; END; END

*.................................................................................................
* L3W class
*.................................................................................................
CLASS L3wText

 DATA aLines       INIT    {}    PROTECTED
 DATA cTxtIn       INIT    ""    PROTECTED
 DATA cTxtOut      INIT    ""    PROTECTED
 DATA cFileName    INIT    ""    PROTECTED
 DATA aRules       INIT    {}    PROTECTED
 DATA lStrip       INIT    .F.   PROTECTED

*     Methods
 METHOD New
 METHOD Eol
 METHOD Reset
 METHOD ArrToTxt
 METHOD TxtToArr
 METHOD Parsing

*     Properties
 METHOD Text       SETGET
 METHOD Fname      SETGET
 METHOD Lines      SETGET
 METHOD Line       SETGET
 METHOD Nline      INLINE LEN( ::aLines )
 METHOD AddRule    SETGET
 METHOD DelRule    SETGET
 METHOD Strip      SETGET

ENDCLASS

*.................................................................................................
* Methods
*.................................................................................................

*.................................................................................................
METHOD New() CLASS L3wText

RETURN Self

*.................................................................................................
METHOD Eol( cString ) CLASS L3wText

 LOCAL nCRPos := AT( CHR( 13 ), cString )
 LOCAL nLFPos := AT( CHR( 10 ), cString )

 IF nCRPos > 0 .AND. nLFPos == 0
  RETURN CHR( 13 )
 ELSEIF nCRPos == 0 .AND. nLFPos >  0
  RETURN CHR( 10 )
 ELSEIF nCRPos > 0 .AND. nLFPos == nCRPos + 1
  RETURN CHR( 13 ) + CHR( 10 )
 ENDIF

RETURN HB_OSNEWLINE()

*.................................................................................................
METHOD Reset() CLASS L3wText
 
 ::cTxtIn    := ""
 ::cTxtOut   := ""
 ::cFileName := ""
 ::aLines    := {}

RETURN NIL

*.................................................................................................
METHOD TxtToArr() CLASS L3wText

 IF LEN( ALLTRIM( ::cTxtIn ) ) = 0 .AND. FILE( ::cFileName )
  ::cTxtIn := MEMOREAD( ::cFileName )
 ENDIF

 ::aLines := HB_ATOKENS( ::cTxtIn, ::Eol( ::cTxtIn ) )

RETURN ::aLines

*.................................................................................................
METHOD ArrToTxt() CLASS L3wText

 LOCAL nX

 ::cTxtOut := ""
 FOR nX := 1 TO LEN( ::aLines )
  ::cTxtOut += ::aLines[nX] + HB_OSNEWLINE()
 NEXT nX

RETURN ::cTxtOut

*.................................................................................................
METHOD Parsing() CLASS L3wText

 LOCAL nX, nAt, cField, cValue

 IF LEN( ALLTRIM( ::cTxtIn ) ) = 0 .AND. FILE( ::cFileName )
  ::cTxtIn := MEMOREAD( ::cFileName )
 ENDIF

 ::cTxtOut := ::cTxtIn

 FOR nX := 1 TO LEN( ::aRules )
  REPEAT
   nAt := AT( ::aRules[nX,1], ::cTxtOut )
   IF nAt <> 0
    IF LEN( ALLTRIM( ::aRules[nX,3] ) ) == 0
     cValue := IF( ::lStrip == .T., ALLTRIM( ::aRules[nX,2] ), ::aRules[nX,2] )
    ELSE
     cField := ::aRules[nX,3] + "->" + ::aRules[nX,2]
     cValue := IF( ::lStrip == .T., ALLTRIM( &(cField) ), &(cField) )
    ENDIF
    ::cTxtOut := STRTRAN( ::cTxtOut, ::aRules[nX,1], cValue )
   ENDIF
  UNTIL nAt == 0
 NEXT nX

RETURN ::cTxtOut

*.................................................................................................
* Properties
*.................................................................................................

*.................................................................................................
METHOD Fname( cFileName ) CLASS L3wText

 IF PCOUNT() == 1
  ::cFileName := cFileName
 ELSE
  RETURN ::cFileName
 ENDIF 

RETURN NIL

*.................................................................................................
METHOD Line( nArg1, cArg2 ) CLASS L3wText

 IF PCOUNT() == 1
  RETURN ::aLines[nArg1]
 ELSE
  ::aLines[nArg1] := cArg2
 ENDIF

RETURN NIL

*.................................................................................................
METHOD Lines( aArg1 ) CLASS L3wText

 IF PCOUNT() == 1
  ::aLines := aArg1
 ELSE
  RETURN ::aLines
 ENDIF

RETURN NIL

*.................................................................................................
METHOD Text( cArg1 ) CLASS L3wText

 IF PCOUNT() == 1
  ::cTxtIn := cArg1
 ELSE
  RETURN ::cTxtIn
 ENDIF

RETURN NIL

*.................................................................................................
METHOD Strip( lArg1 ) CLASS L3wText

 IF PCOUNT() == 1
  ::lStrip := lArg1
 ELSE
  RETURN ::lStrip
 ENDIF

RETURN NIL

*.................................................................................................
METHOD AddRule( cArg1, cArg2, cArg3 ) CLASS L3wText

 LOCAL nX, aReturn

 cArg1 := UPPER( ALLTRIM( cArg1 ) )
 cArg3 := IF( VALTYPE( cArg3 ) == "C", cArg3, "" )
 nX := ASCAN( ::aRules, { |x| x[1]==cArg1 } )

 IF PCOUNT() >= 2
  IF nX == 0
   AADD( ::aRules, { cArg1, cArg2, cArg3 } )
   nX := LEN( ::aRules )
  ENDIF
  ::aRules[nX,1] := cArg1
  ::aRules[nX,2] := cArg2
  ::aRules[nX,3] := cArg3
 ELSE
  aReturn := IF( nX > 0, { ::aRules[nX,2], ::aRules[nX,2] }, {} )
  RETURN aReturn
 ENDIF

RETURN NIL

*.................................................................................................
METHOD DelRule( cArg1 ) CLASS L3wText

 LOCAL nX

 cArg1 := UPPER( ALLTRIM( cArg1 ) )

 nX := ASCAN( ::aRules, { |x| x[1]==cArg1 } )

 IF nX <> 0
  HB_ADEL( ::aRules, nX, .T. )
 ENDIF

RETURN NIL
